@@ -32,18 +32,6 @@ struct d2d_draw_text_layout_ctx
     D2D1_DRAW_TEXT_OPTIONS options;
 };
 
-static void d2d_matrix_multiply(D2D_MATRIX_3X2_F *a, const D2D_MATRIX_3X2_F *b)
-{
-    D2D_MATRIX_3X2_F tmp = *a;
-
-    a->_11 = tmp._11 * b->_11 + tmp._12 * b->_21;
-    a->_12 = tmp._11 * b->_12 + tmp._12 * b->_22;
-    a->_21 = tmp._21 * b->_11 + tmp._22 * b->_21;
-    a->_22 = tmp._21 * b->_12 + tmp._22 * b->_22;
-    a->_31 = tmp._31 * b->_11 + tmp._32 * b->_21 + b->_31;
-    a->_32 = tmp._31 * b->_12 + tmp._32 * b->_22 + b->_32;
-}
-
 static void d2d_point_set(D2D1_POINT_2F *dst, float x, float y)
 {
     dst->x = x;
@@ -139,14 +127,15 @@ static void d2d_clip_stack_pop(struct d2d_clip_stack *stack)
     --stack->count;
 }
 
-static void d2d_draw(struct d2d_d3d_render_target *render_target, ID3D10Buffer *vs_cb,
-        ID3D10PixelShader *ps, ID3D10Buffer *ps_cb, struct d2d_brush *brush)
+static void d2d_draw(struct d2d_d3d_render_target *render_target, enum d2d_shape_type shape_type,
+        ID3D10Buffer *ib, unsigned int index_count, ID3D10Buffer *vb, unsigned int vb_stride,
+        ID3D10Buffer *vs_cb, ID3D10Buffer *ps_cb, struct d2d_brush *brush)
 {
+    struct d2d_shape_resources *shape_resources = &render_target->shape_resources[shape_type];
     ID3D10Device *device = render_target->device;
     unsigned int offset;
     D3D10_VIEWPORT vp;
     HRESULT hr;
-    static const float blend_factor[] = {1.0f, 1.0f, 1.0f, 1.0f};
 
     vp.TopLeftX = 0;
     vp.TopLeftY = 0;
@@ -163,15 +152,14 @@ static void d2d_draw(struct d2d_d3d_render_target *render_target, ID3D10Buffer *
 
     ID3D10Device_ClearState(device);
 
-    ID3D10Device_IASetInputLayout(device, render_target->il);
-    ID3D10Device_IASetPrimitiveTopology(device, D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP);
+    ID3D10Device_IASetInputLayout(device, shape_resources->il);
+    ID3D10Device_IASetPrimitiveTopology(device, D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
+    ID3D10Device_IASetIndexBuffer(device, ib, DXGI_FORMAT_R16_UINT, 0);
     offset = 0;
-    ID3D10Device_IASetVertexBuffers(device, 0, 1, &render_target->vb,
-            &render_target->vb_stride, &offset);
+    ID3D10Device_IASetVertexBuffers(device, 0, 1, &vb, &vb_stride, &offset);
     ID3D10Device_VSSetConstantBuffers(device, 0, 1, &vs_cb);
-    ID3D10Device_VSSetShader(device, render_target->vs);
+    ID3D10Device_VSSetShader(device, shape_resources->vs);
     ID3D10Device_PSSetConstantBuffers(device, 0, 1, &ps_cb);
-    ID3D10Device_PSSetShader(device, ps);
     ID3D10Device_RSSetViewports(device, 1, &vp);
     if (render_target->clip_stack.count)
     {
@@ -188,12 +176,14 @@ static void d2d_draw(struct d2d_d3d_render_target *render_target, ID3D10Buffer *
     }
     ID3D10Device_OMSetRenderTargets(device, 1, &render_target->view, NULL);
     if (brush)
-    {
-        ID3D10Device_OMSetBlendState(device, render_target->bs, blend_factor, D3D10_DEFAULT_SAMPLE_MASK);
-        d2d_brush_bind_resources(brush, device);
-    }
+        d2d_brush_bind_resources(brush, render_target, shape_type);
+    else
+        ID3D10Device_PSSetShader(device, shape_resources->ps[D2D_BRUSH_TYPE_SOLID]);
 
-    ID3D10Device_Draw(device, 4, 0);
+    if (ib)
+        ID3D10Device_DrawIndexed(device, index_count, 0, 0);
+    else
+        ID3D10Device_Draw(device, index_count, 0);
 
     if (FAILED(hr = render_target->stateblock->lpVtbl->Apply(render_target->stateblock)))
         WARN("Failed to apply stateblock, hr %#x.\n", hr);
@@ -242,16 +232,25 @@ static ULONG STDMETHODCALLTYPE d2d_d3d_render_target_Release(ID2D1RenderTarget *
 
     if (!refcount)
     {
+        unsigned int i, j;
+
         d2d_clip_stack_cleanup(&render_target->clip_stack);
         if (render_target->text_rendering_params)
             IDWriteRenderingParams_Release(render_target->text_rendering_params);
-        ID3D10PixelShader_Release(render_target->rect_bitmap_ps);
-        ID3D10PixelShader_Release(render_target->rect_solid_ps);
         ID3D10BlendState_Release(render_target->bs);
         ID3D10RasterizerState_Release(render_target->rs);
-        ID3D10VertexShader_Release(render_target->vs);
         ID3D10Buffer_Release(render_target->vb);
-        ID3D10InputLayout_Release(render_target->il);
+        ID3D10Buffer_Release(render_target->ib);
+        for (i = 0; i < D2D_SHAPE_TYPE_COUNT; ++i)
+        {
+            for (j = 0; j < D2D_BRUSH_TYPE_COUNT; ++j)
+            {
+                if (render_target->shape_resources[i].ps[j])
+                    ID3D10PixelShader_Release(render_target->shape_resources[i].ps[j]);
+            }
+            ID3D10VertexShader_Release(render_target->shape_resources[i].vs);
+            ID3D10InputLayout_Release(render_target->shape_resources[i].il);
+        }
         render_target->stateblock->lpVtbl->Release(render_target->stateblock);
         ID3D10RenderTargetView_Release(render_target->view);
         ID3D10Device_Release(render_target->device);
@@ -301,14 +300,28 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateBitmap(ID2D1RenderT
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateBitmapFromWicBitmap(ID2D1RenderTarget *iface,
         IWICBitmapSource *bitmap_source, const D2D1_BITMAP_PROPERTIES *desc, ID2D1Bitmap **bitmap)
 {
+    const D2D1_PIXEL_FORMAT *d2d_format;
     D2D1_BITMAP_PROPERTIES bitmap_desc;
+    WICPixelFormatGUID wic_format;
     unsigned int bpp, data_size;
     D2D1_SIZE_U size;
+    unsigned int i;
     WICRect rect;
     UINT32 pitch;
     HRESULT hr;
     void *data;
 
+    static const struct
+    {
+        const WICPixelFormatGUID *wic;
+        D2D1_PIXEL_FORMAT d2d;
+    }
+    format_lookup[] =
+    {
+        {&GUID_WICPixelFormat32bppPBGRA, {DXGI_FORMAT_B8G8R8A8_UNORM, D2D1_ALPHA_MODE_PREMULTIPLIED}},
+        {&GUID_WICPixelFormat32bppBGR,   {DXGI_FORMAT_B8G8R8A8_UNORM, D2D1_ALPHA_MODE_IGNORE}},
+    };
+
     TRACE("iface %p, bitmap_source %p, desc %p, bitmap %p.\n",
             iface, bitmap_source, desc, bitmap);
 
@@ -330,28 +343,32 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateBitmapFromWicBitmap
         bitmap_desc = *desc;
     }
 
-    if (bitmap_desc.pixelFormat.format == DXGI_FORMAT_UNKNOWN)
+    if (FAILED(hr = IWICBitmapSource_GetPixelFormat(bitmap_source, &wic_format)))
     {
-        WICPixelFormatGUID wic_format;
+        WARN("Failed to get bitmap format, hr %#x.\n", hr);
+        return hr;
+    }
 
-        if (FAILED(hr = IWICBitmapSource_GetPixelFormat(bitmap_source, &wic_format)))
+    for (i = 0, d2d_format = NULL; i < sizeof(format_lookup) / sizeof(*format_lookup); ++i)
+    {
+        if (IsEqualGUID(&wic_format, format_lookup[i].wic))
         {
-            WARN("Failed to get bitmap format, hr %#x.\n", hr);
-            return hr;
-        }
-
-        if (IsEqualGUID(&wic_format, &GUID_WICPixelFormat32bppPBGRA)
-                || IsEqualGUID(&wic_format, &GUID_WICPixelFormat32bppBGR))
-        {
-            bitmap_desc.pixelFormat.format = DXGI_FORMAT_B8G8R8A8_UNORM;
-        }
-        else
-        {
-            WARN("Unsupported WIC bitmap format %s.\n", debugstr_guid(&wic_format));
-            return D2DERR_UNSUPPORTED_PIXEL_FORMAT;
+            d2d_format = &format_lookup[i].d2d;
+            break;
         }
     }
 
+    if (!d2d_format)
+    {
+        WARN("Unsupported WIC bitmap format %s.\n", debugstr_guid(&wic_format));
+        return D2DERR_UNSUPPORTED_PIXEL_FORMAT;
+    }
+
+    if (bitmap_desc.pixelFormat.format == DXGI_FORMAT_UNKNOWN)
+        bitmap_desc.pixelFormat.format = d2d_format->format;
+    if (bitmap_desc.pixelFormat.alphaMode == D2D1_ALPHA_MODE_UNKNOWN)
+        bitmap_desc.pixelFormat.alphaMode = d2d_format->alphaMode;
+
     switch (bitmap_desc.pixelFormat.format)
     {
         case DXGI_FORMAT_B8G8R8A8_UNORM:
@@ -547,14 +564,91 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawRectangle(ID2D1RenderTar
 
 static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRectangle(ID2D1RenderTarget *iface,
         const D2D1_RECT_F *rect, ID2D1Brush *brush)
+{
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    ID2D1RectangleGeometry *geometry;
+    HRESULT hr;
+
+    TRACE("iface %p, rect %p, brush %p.\n", iface, rect, brush);
+
+    if (FAILED(hr = ID2D1Factory_CreateRectangleGeometry(render_target->factory, rect, &geometry)))
+    {
+        ERR("Failed to create geometry, hr %#x.\n", hr);
+        return;
+    }
+
+    ID2D1RenderTarget_FillGeometry(iface, (ID2D1Geometry *)geometry, brush, NULL);
+    ID2D1RectangleGeometry_Release(geometry);
+}
+
+static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawRoundedRectangle(ID2D1RenderTarget *iface,
+        const D2D1_ROUNDED_RECT *rect, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
+{
+    FIXME("iface %p, rect %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
+            iface, rect, brush, stroke_width, stroke_style);
+}
+
+static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRoundedRectangle(ID2D1RenderTarget *iface,
+        const D2D1_ROUNDED_RECT *rect, ID2D1Brush *brush)
+{
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    ID2D1RoundedRectangleGeometry *geometry;
+    HRESULT hr;
+
+    TRACE("iface %p, rect %p, brush %p.\n", iface, rect, brush);
+
+    if (FAILED(hr = ID2D1Factory_CreateRoundedRectangleGeometry(render_target->factory, rect, &geometry)))
+    {
+        ERR("Failed to create geometry, hr %#x.\n", hr);
+        return;
+    }
+
+    ID2D1RenderTarget_FillGeometry(iface, (ID2D1Geometry *)geometry, brush, NULL);
+    ID2D1RoundedRectangleGeometry_Release(geometry);
+}
+
+static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawEllipse(ID2D1RenderTarget *iface,
+        const D2D1_ELLIPSE *ellipse, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
+{
+    FIXME("iface %p, ellipse %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
+            iface, ellipse, brush, stroke_width, stroke_style);
+}
+
+static void STDMETHODCALLTYPE d2d_d3d_render_target_FillEllipse(ID2D1RenderTarget *iface,
+        const D2D1_ELLIPSE *ellipse, ID2D1Brush *brush)
+{
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    ID2D1EllipseGeometry *geometry;
+    HRESULT hr;
+
+    TRACE("iface %p, ellipse %p, brush %p.\n", iface, ellipse, brush);
+
+    if (FAILED(hr = ID2D1Factory_CreateEllipseGeometry(render_target->factory, ellipse, &geometry)))
+    {
+        ERR("Failed to create geometry, hr %#x.\n", hr);
+        return;
+    }
+
+    ID2D1RenderTarget_FillGeometry(iface, (ID2D1Geometry *)geometry, brush, NULL);
+    ID2D1EllipseGeometry_Release(geometry);
+}
+
+static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawGeometry(ID2D1RenderTarget *iface,
+        ID2D1Geometry *geometry, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
+{
+    FIXME("iface %p, geometry %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
+            iface, geometry, brush, stroke_width, stroke_style);
+}
+
+static void STDMETHODCALLTYPE d2d_d3d_render_target_FillGeometry(ID2D1RenderTarget *iface,
+        ID2D1Geometry *geometry, ID2D1Brush *brush, ID2D1Brush *opacity_brush)
 {
     struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
     struct d2d_brush *brush_impl = unsafe_impl_from_ID2D1Brush(brush);
+    const struct d2d_geometry *geometry_impl;
+    ID3D10Buffer *ib, *vb, *vs_cb, *ps_cb;
     D3D10_SUBRESOURCE_DATA buffer_data;
     D3D10_BUFFER_DESC buffer_desc;
-    ID3D10Buffer *vs_cb, *ps_cb;
-    ID3D10PixelShader *ps;
-    D2D1_COLOR_F color;
     float tmp_x, tmp_y;
     HRESULT hr;
     struct
@@ -563,17 +657,13 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRectangle(ID2D1RenderTar
         float _12, _22, _32, pad1;
     } transform;
 
-    TRACE("iface %p, rect %p, brush %p.\n", iface, rect, brush);
+    TRACE("iface %p, geometry %p, brush %p, opacity_brush %p.\n", iface, geometry, brush, opacity_brush);
 
-    if (brush_impl->type != D2D_BRUSH_TYPE_SOLID
-            && brush_impl->type != D2D_BRUSH_TYPE_BITMAP)
-    {
-        FIXME("Unhandled brush type %#x.\n", brush_impl->type);
-        return;
-    }
+    if (opacity_brush)
+        FIXME("Ignoring opacity brush %p.\n", opacity_brush);
+
+    geometry_impl = unsafe_impl_from_ID2D1Geometry(geometry);
 
-    /* Translate from clip space to world (D2D rendertarget) space, taking the
-     * dpi and rendertarget transform into account. */
     tmp_x =  (2.0f * render_target->dpi_x) / (96.0f * render_target->pixel_size.width);
     tmp_y = -(2.0f * render_target->dpi_y) / (96.0f * render_target->pixel_size.height);
     transform._11 = render_target->drawing_state.transform._11 * tmp_x;
@@ -585,18 +675,6 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRectangle(ID2D1RenderTar
     transform._32 = render_target->drawing_state.transform._32 * tmp_y + 1.0f;
     transform.pad1 = 0.0f;
 
-    /* Translate from world space to object space. */
-    tmp_x = min(rect->left, rect->right) + fabsf(rect->right - rect->left) / 2.0f;
-    tmp_y = min(rect->top, rect->bottom) + fabsf(rect->bottom - rect->top) / 2.0f;
-    transform._31 += tmp_x * transform._11 + tmp_y * transform._21;
-    transform._32 += tmp_x * transform._12 + tmp_y * transform._22;
-    tmp_x = fabsf(rect->right - rect->left) / 2.0f;
-    tmp_y = fabsf(rect->bottom - rect->top) / 2.0f;
-    transform._11 *= tmp_x;
-    transform._12 *= tmp_x;
-    transform._21 *= tmp_y;
-    transform._22 *= tmp_y;
-
     buffer_desc.ByteWidth = sizeof(transform);
     buffer_desc.Usage = D3D10_USAGE_DEFAULT;
     buffer_desc.BindFlags = D3D10_BIND_CONSTANT_BUFFER;
@@ -613,117 +691,62 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRectangle(ID2D1RenderTar
         return;
     }
 
-    if (brush_impl->type == D2D_BRUSH_TYPE_BITMAP)
+    if (FAILED(hr = d2d_brush_get_ps_cb(brush_impl, render_target, &ps_cb)))
     {
-        struct d2d_bitmap *bitmap = brush_impl->u.bitmap.bitmap;
-        D2D_MATRIX_3X2_F w, b;
-        float dpi_scale, d;
-
-        ps = render_target->rect_bitmap_ps;
-
-        /* Scale for dpi. */
-        w = render_target->drawing_state.transform;
-        dpi_scale = render_target->dpi_x / 96.0f;
-        w._11 *= dpi_scale;
-        w._21 *= dpi_scale;
-        w._31 *= dpi_scale;
-        dpi_scale = render_target->dpi_y / 96.0f;
-        w._12 *= dpi_scale;
-        w._22 *= dpi_scale;
-        w._32 *= dpi_scale;
-
-        /* Scale for bitmap size and dpi. */
-        b = brush_impl->transform;
-        dpi_scale = bitmap->pixel_size.width * (bitmap->dpi_x / 96.0f);
-        b._11 *= dpi_scale;
-        b._21 *= dpi_scale;
-        dpi_scale = bitmap->pixel_size.height * (bitmap->dpi_y / 96.0f);
-        b._12 *= dpi_scale;
-        b._22 *= dpi_scale;
-
-        d2d_matrix_multiply(&b, &w);
-
-        /* Invert the matrix. (Because the matrix is applied to the sampling
-         * coordinates. I.e., to scale the bitmap by 2 we need to divide the
-         * coordinates by 2.) */
-        d = b._11 * b._22 - b._21 * b._12;
-        if (d != 0.0f)
-        {
-            transform._11 = b._22 / d;
-            transform._21 = -b._21 / d;
-            transform._31 = (b._21 * b._32 - b._31 * b._22) / d;
-            transform._12 = -b._12 / d;
-            transform._22 = b._11 / d;
-            transform._32 = -(b._11 * b._32 - b._31 * b._12) / d;
-        }
-        transform.pad1 = brush_impl->opacity;
-
-        buffer_desc.ByteWidth = sizeof(transform);
-        buffer_data.pSysMem = &transform;
-    }
-    else
-    {
-        ps = render_target->rect_solid_ps;
-
-        color = brush_impl->u.solid.color;
-        color.a *= brush_impl->opacity;
-
-        buffer_desc.ByteWidth = sizeof(color);
-        buffer_data.pSysMem = &color;
-    }
-
-    if (FAILED(hr = ID3D10Device_CreateBuffer(render_target->device, &buffer_desc, &buffer_data, &ps_cb)))
-    {
-        WARN("Failed to create constant buffer, hr %#x.\n", hr);
+        WARN("Failed to get ps constant buffer, hr %#x.\n", hr);
         ID3D10Buffer_Release(vs_cb);
         return;
     }
 
-    d2d_draw(render_target, vs_cb, ps, ps_cb, brush_impl);
+    buffer_desc.ByteWidth = geometry_impl->face_count * sizeof(*geometry_impl->faces);
+    buffer_desc.BindFlags = D3D10_BIND_INDEX_BUFFER;
+    buffer_data.pSysMem = geometry_impl->faces;
 
+    if (FAILED(hr = ID3D10Device_CreateBuffer(render_target->device, &buffer_desc, &buffer_data, &ib)))
+    {
+        WARN("Failed to create index buffer, hr %#x.\n", hr);
+        goto done;
+    }
+
+    buffer_desc.ByteWidth = geometry_impl->vertex_count * sizeof(*geometry_impl->vertices);
+    buffer_desc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
+    buffer_data.pSysMem = geometry_impl->vertices;
+
+    if (FAILED(hr = ID3D10Device_CreateBuffer(render_target->device, &buffer_desc, &buffer_data, &vb)))
+    {
+        ERR("Failed to create vertex buffer, hr %#x.\n", hr);
+        ID3D10Buffer_Release(ib);
+        goto done;
+    }
+
+    d2d_draw(render_target, D2D_SHAPE_TYPE_TRIANGLE, ib, 3 * geometry_impl->face_count, vb,
+            sizeof(*geometry_impl->vertices), vs_cb, ps_cb, brush_impl);
+
+    ID3D10Buffer_Release(vb);
+    ID3D10Buffer_Release(ib);
+
+    if (geometry_impl->bezier_count)
+    {
+        buffer_desc.ByteWidth = geometry_impl->bezier_count * sizeof(*geometry_impl->beziers);
+        buffer_data.pSysMem = geometry_impl->beziers;
+
+        if (FAILED(hr = ID3D10Device_CreateBuffer(render_target->device, &buffer_desc, &buffer_data, &vb)))
+        {
+            ERR("Failed to create beziers vertex buffer, hr %#x.\n", hr);
+            goto done;
+        }
+
+        d2d_draw(render_target, D2D_SHAPE_TYPE_BEZIER, NULL, 3 * geometry_impl->bezier_count, vb,
+                sizeof(*geometry_impl->beziers->v), vs_cb, ps_cb, brush_impl);
+
+        ID3D10Buffer_Release(vb);
+    }
+
+done:
     ID3D10Buffer_Release(ps_cb);
     ID3D10Buffer_Release(vs_cb);
 }
 
-static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawRoundedRectangle(ID2D1RenderTarget *iface,
-        const D2D1_ROUNDED_RECT *rect, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
-{
-    FIXME("iface %p, rect %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
-            iface, rect, brush, stroke_width, stroke_style);
-}
-
-static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRoundedRectangle(ID2D1RenderTarget *iface,
-        const D2D1_ROUNDED_RECT *rect, ID2D1Brush *brush)
-{
-    FIXME("iface %p, rect %p, brush %p stub!\n", iface, rect, brush);
-}
-
-static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawEllipse(ID2D1RenderTarget *iface,
-        const D2D1_ELLIPSE *ellipse, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
-{
-    FIXME("iface %p, ellipse %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
-            iface, ellipse, brush, stroke_width, stroke_style);
-}
-
-static void STDMETHODCALLTYPE d2d_d3d_render_target_FillEllipse(ID2D1RenderTarget *iface,
-        const D2D1_ELLIPSE *ellipse, ID2D1Brush *brush)
-{
-    FIXME("iface %p, ellipse %p, brush %p stub!\n", iface, ellipse, brush);
-}
-
-static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawGeometry(ID2D1RenderTarget *iface,
-        ID2D1Geometry *geometry, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
-{
-    FIXME("iface %p, geometry %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
-            iface, geometry, brush, stroke_width, stroke_style);
-}
-
-static void STDMETHODCALLTYPE d2d_d3d_render_target_FillGeometry(ID2D1RenderTarget *iface,
-        ID2D1Geometry *geometry, ID2D1Brush *brush, ID2D1Brush *opacity_brush)
-{
-    FIXME("iface %p, geometry %p, brush %p, opacity_brush %p stub!\n", iface, geometry, brush, opacity_brush);
-}
-
 static void STDMETHODCALLTYPE d2d_d3d_render_target_FillMesh(ID2D1RenderTarget *iface,
         ID2D1Mesh *mesh, ID2D1Brush *brush)
 {
@@ -871,6 +894,10 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawGlyphRun(ID2D1RenderTarg
 
     if (measuring_mode)
         FIXME("Ignoring measuring mode %#x.\n", measuring_mode);
+    if (render_target->text_rendering_params)
+        FIXME("Ignoring text rendering parameters %p.\n", render_target->text_rendering_params);
+    if (render_target->drawing_state.textAntialiasMode != D2D1_TEXT_ANTIALIAS_MODE_ALIASED)
+        FIXME("Ignoring text antialiasing mode %#x.\n", render_target->drawing_state.textAntialiasMode);
 
     if (FAILED(hr = ID2D1Factory_CreatePathGeometry(render_target->factory, &geometry)))
     {
@@ -1145,7 +1172,8 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_Clear(ID2D1RenderTarget *ifa
         return;
     }
 
-    d2d_draw(render_target, vs_cb, render_target->rect_solid_ps, ps_cb, NULL);
+    d2d_draw(render_target, D2D_SHAPE_TYPE_TRIANGLE, render_target->ib, 6,
+            render_target->vb, render_target->vb_stride, vs_cb, ps_cb, NULL);
 
     ID3D10Buffer_Release(ps_cb);
     ID3D10Buffer_Release(vs_cb);
@@ -1172,10 +1200,11 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_EndDraw(ID2D1RenderTarget
 static D2D1_PIXEL_FORMAT * STDMETHODCALLTYPE d2d_d3d_render_target_GetPixelFormat(ID2D1RenderTarget *iface,
         D2D1_PIXEL_FORMAT *format)
 {
-    FIXME("iface %p, format %p stub!\n", iface, format);
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
 
-    format->format = DXGI_FORMAT_UNKNOWN;
-    format->alphaMode = D2D1_ALPHA_MODE_UNKNOWN;
+    TRACE("iface %p, format %p.\n", iface, format);
+
+    *format = render_target->format;
     return format;
 }
 
@@ -1458,13 +1487,19 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
     D3D10_BUFFER_DESC buffer_desc;
     D3D10_BLEND_DESC blend_desc;
     ID3D10Resource *resource;
+    unsigned int i, j;
     HRESULT hr;
 
-    static const D3D10_INPUT_ELEMENT_DESC il_desc[] =
+    static const D3D10_INPUT_ELEMENT_DESC il_desc_triangle[] =
     {
         {"POSITION", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0},
     };
-    static const DWORD vs_code[] =
+    static const D3D10_INPUT_ELEMENT_DESC il_desc_bezier[] =
+    {
+        {"POSITION", 0, DXGI_FORMAT_R32G32_FLOAT,    0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0},
+        {"TEXCOORD", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 8, D3D10_INPUT_PER_VERTEX_DATA, 0},
+    };
+    static const DWORD vs_code_triangle[] =
     {
         /* float3x2 transform;
          *
@@ -1483,7 +1518,32 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         0x00000000, 0x00208246, 0x00000000, 0x00000001, 0x05000036, 0x001020c2, 0x00000000, 0x00101ea6,
         0x00000000, 0x0100003e,
     };
-    static const DWORD rect_solid_ps_code[] =
+    static const DWORD vs_code_bezier[] =
+    {
+#if 0
+        float3x2 transform;
+
+        float4 main(float4 position : POSITION,
+              inout float3 texcoord : TEXCOORD0) : SV_POSITION
+        {
+            return float4(mul(position.xyw, transform), position.zw);
+        }
+#endif
+        0x43425844, 0x5e578adb, 0x093f7e27, 0x50d478af, 0xec3dfa4f, 0x00000001, 0x00000198, 0x00000003,
+        0x0000002c, 0x00000080, 0x000000d8, 0x4e475349, 0x0000004c, 0x00000002, 0x00000008, 0x00000038,
+        0x00000000, 0x00000000, 0x00000003, 0x00000000, 0x00000f0f, 0x00000041, 0x00000000, 0x00000000,
+        0x00000003, 0x00000001, 0x00000707, 0x49534f50, 0x4e4f4954, 0x58455400, 0x524f4f43, 0xabab0044,
+        0x4e47534f, 0x00000050, 0x00000002, 0x00000008, 0x00000038, 0x00000000, 0x00000001, 0x00000003,
+        0x00000000, 0x0000000f, 0x00000044, 0x00000000, 0x00000000, 0x00000003, 0x00000001, 0x00000807,
+        0x505f5653, 0x5449534f, 0x004e4f49, 0x43584554, 0x44524f4f, 0xababab00, 0x52444853, 0x000000b8,
+        0x00010040, 0x0000002e, 0x04000059, 0x00208e46, 0x00000000, 0x00000002, 0x0300005f, 0x001010f2,
+        0x00000000, 0x0300005f, 0x00101072, 0x00000001, 0x04000067, 0x001020f2, 0x00000000, 0x00000001,
+        0x03000065, 0x00102072, 0x00000001, 0x08000010, 0x00102012, 0x00000000, 0x00101346, 0x00000000,
+        0x00208246, 0x00000000, 0x00000000, 0x08000010, 0x00102022, 0x00000000, 0x00101346, 0x00000000,
+        0x00208246, 0x00000000, 0x00000001, 0x05000036, 0x001020c2, 0x00000000, 0x00101ea6, 0x00000000,
+        0x05000036, 0x00102072, 0x00000001, 0x00101246, 0x00000001, 0x0100003e,
+    };
+    static const DWORD ps_code_triangle_solid[] =
     {
         /* float4 color;
          *
@@ -1499,7 +1559,7 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         0x00000010, 0x04000059, 0x00208e46, 0x00000000, 0x00000001, 0x03000065, 0x001020f2, 0x00000000,
         0x06000036, 0x001020f2, 0x00000000, 0x00208e46, 0x00000000, 0x00000000, 0x0100003e,
     };
-    static const DWORD rect_bitmap_ps_code[] =
+    static const DWORD ps_code_triangle_bitmap[] =
     {
 #if 0
         float3x2 transform;
@@ -1537,6 +1597,32 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         0x00000000, 0x0010003a, 0x00000000, 0x0020803a, 0x00000000, 0x00000001, 0x05000036, 0x00102072,
         0x00000000, 0x00100246, 0x00000000, 0x0100003e,
     };
+    /* The basic idea here is to evaluate the implicit form of the curve in
+     * texture space. "t.z" determines which side of the curve is shaded. */
+    static const DWORD ps_code_bezier_solid[] =
+    {
+#if 0
+        float4 color;
+
+        float4 main(float4 position : SV_POSITION, float3 t : TEXCOORD0) : SV_Target
+        {
+            clip((t.x * t.x - t.y) * t.z);
+            return color;
+        }
+#endif
+        0x43425844, 0x66075f9e, 0x2ffe405b, 0xb551ee63, 0xa0d9f457, 0x00000001, 0x00000180, 0x00000003,
+        0x0000002c, 0x00000084, 0x000000b8, 0x4e475349, 0x00000050, 0x00000002, 0x00000008, 0x00000038,
+        0x00000000, 0x00000001, 0x00000003, 0x00000000, 0x0000000f, 0x00000044, 0x00000000, 0x00000000,
+        0x00000003, 0x00000001, 0x00000707, 0x505f5653, 0x5449534f, 0x004e4f49, 0x43584554, 0x44524f4f,
+        0xababab00, 0x4e47534f, 0x0000002c, 0x00000001, 0x00000008, 0x00000020, 0x00000000, 0x00000000,
+        0x00000003, 0x00000000, 0x0000000f, 0x545f5653, 0x65677261, 0xabab0074, 0x52444853, 0x000000c0,
+        0x00000040, 0x00000030, 0x04000059, 0x00208e46, 0x00000000, 0x00000001, 0x03001062, 0x00101072,
+        0x00000001, 0x03000065, 0x001020f2, 0x00000000, 0x02000068, 0x00000001, 0x0a000032, 0x00100012,
+        0x00000000, 0x0010100a, 0x00000001, 0x0010100a, 0x00000001, 0x8010101a, 0x00000041, 0x00000001,
+        0x07000038, 0x00100012, 0x00000000, 0x0010000a, 0x00000000, 0x0010102a, 0x00000001, 0x07000031,
+        0x00100012, 0x00000000, 0x0010000a, 0x00000000, 0x00004001, 0x00000000, 0x0304000d, 0x0010000a,
+        0x00000000, 0x06000036, 0x001020f2, 0x00000000, 0x00208e46, 0x00000000, 0x00000000, 0x0100003e,
+    };
     static const struct
     {
         float x, y;
@@ -1548,6 +1634,7 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         { 1.0f,  1.0f},
         { 1.0f, -1.0f},
     };
+    static const UINT16 indices[] = {0, 1, 2, 2, 1, 3};
     static const D2D1_MATRIX_3X2_F identity =
     {
         1.0f, 0.0f,
@@ -1596,23 +1683,80 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         goto err;
     }
 
-    if (FAILED(hr = ID3D10Device_CreateInputLayout(render_target->device, il_desc,
-            sizeof(il_desc) / sizeof(*il_desc), vs_code, sizeof(vs_code),
-            &render_target->il)))
+    if (FAILED(hr = ID3D10Device_CreateInputLayout(render_target->device, il_desc_triangle,
+            sizeof(il_desc_triangle) / sizeof(*il_desc_triangle), vs_code_triangle, sizeof(vs_code_triangle),
+            &render_target->shape_resources[D2D_SHAPE_TYPE_TRIANGLE].il)))
     {
-        WARN("Failed to create clear input layout, hr %#x.\n", hr);
+        WARN("Failed to create triangle input layout, hr %#x.\n", hr);
+        goto err;
+    }
+
+    if (FAILED(hr = ID3D10Device_CreateInputLayout(render_target->device, il_desc_bezier,
+            sizeof(il_desc_bezier) / sizeof(*il_desc_bezier), vs_code_bezier, sizeof(vs_code_bezier),
+            &render_target->shape_resources[D2D_SHAPE_TYPE_BEZIER].il)))
+    {
+        WARN("Failed to create bezier input layout, hr %#x.\n", hr);
+        goto err;
+    }
+
+    if (FAILED(hr = ID3D10Device_CreateVertexShader(render_target->device, vs_code_triangle,
+            sizeof(vs_code_triangle), &render_target->shape_resources[D2D_SHAPE_TYPE_TRIANGLE].vs)))
+    {
+        WARN("Failed to create triangle vertex shader, hr %#x.\n", hr);
+        goto err;
+    }
+
+    if (FAILED(hr = ID3D10Device_CreateVertexShader(render_target->device,  vs_code_bezier,
+            sizeof(vs_code_bezier), &render_target->shape_resources[D2D_SHAPE_TYPE_BEZIER].vs)))
+    {
+        WARN("Failed to create bezier vertex shader, hr %#x.\n", hr);
+        goto err;
+    }
+
+    if (FAILED(hr = ID3D10Device_CreatePixelShader(render_target->device,
+            ps_code_triangle_solid, sizeof(ps_code_triangle_solid),
+            &render_target->shape_resources[D2D_SHAPE_TYPE_TRIANGLE].ps[D2D_BRUSH_TYPE_SOLID])))
+    {
+        WARN("Failed to create triangle/solid pixel shader, hr %#x.\n", hr);
+        goto err;
+    }
+
+    if (FAILED(hr = ID3D10Device_CreatePixelShader(render_target->device,
+            ps_code_triangle_bitmap, sizeof(ps_code_triangle_bitmap),
+            &render_target->shape_resources[D2D_SHAPE_TYPE_TRIANGLE].ps[D2D_BRUSH_TYPE_BITMAP])))
+    {
+        WARN("Failed to create triangle/bitmap pixel shader, hr %#x.\n", hr);
+        goto err;
+    }
+
+    if (FAILED(hr = ID3D10Device_CreatePixelShader(render_target->device,
+            ps_code_bezier_solid, sizeof(ps_code_bezier_solid),
+            &render_target->shape_resources[D2D_SHAPE_TYPE_BEZIER].ps[D2D_BRUSH_TYPE_SOLID])))
+    {
+        WARN("Failed to create bezier/solid pixel shader, hr %#x.\n", hr);
+        goto err;
+    }
+
+    buffer_desc.ByteWidth = sizeof(indices);
+    buffer_desc.Usage = D3D10_USAGE_DEFAULT;
+    buffer_desc.BindFlags = D3D10_BIND_INDEX_BUFFER;
+    buffer_desc.CPUAccessFlags = 0;
+    buffer_desc.MiscFlags = 0;
+
+    buffer_data.pSysMem = indices;
+    buffer_data.SysMemPitch = 0;
+    buffer_data.SysMemSlicePitch = 0;
+
+    if (FAILED(hr = ID3D10Device_CreateBuffer(render_target->device,
+            &buffer_desc, &buffer_data, &render_target->ib)))
+    {
+        WARN("Failed to create clear index buffer, hr %#x.\n", hr);
         goto err;
     }
 
     buffer_desc.ByteWidth = sizeof(quad);
-    buffer_desc.Usage = D3D10_USAGE_DEFAULT;
     buffer_desc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
-    buffer_desc.CPUAccessFlags = 0;
-    buffer_desc.MiscFlags = 0;
-
     buffer_data.pSysMem = quad;
-    buffer_data.SysMemPitch = 0;
-    buffer_data.SysMemSlicePitch = 0;
 
     render_target->vb_stride = sizeof(*quad);
     if (FAILED(hr = ID3D10Device_CreateBuffer(render_target->device,
@@ -1622,13 +1766,6 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         goto err;
     }
 
-    if (FAILED(hr = ID3D10Device_CreateVertexShader(render_target->device,
-            vs_code, sizeof(vs_code), &render_target->vs)))
-    {
-        WARN("Failed to create clear vertex shader, hr %#x.\n", hr);
-        goto err;
-    }
-
     rs_desc.FillMode = D3D10_FILL_SOLID;
     rs_desc.CullMode = D3D10_CULL_BACK;
     rs_desc.FrontCounterClockwise = FALSE;
@@ -1660,26 +1797,13 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         goto err;
     }
 
-    if (FAILED(hr = ID3D10Device_CreatePixelShader(render_target->device,
-            rect_solid_ps_code, sizeof(rect_solid_ps_code), &render_target->rect_solid_ps)))
-    {
-        WARN("Failed to create pixel shader, hr %#x.\n", hr);
-        goto err;
-    }
-
-    if (FAILED(hr = ID3D10Device_CreatePixelShader(render_target->device,
-            rect_bitmap_ps_code, sizeof(rect_bitmap_ps_code), &render_target->rect_bitmap_ps)))
-    {
-        WARN("Failed to create pixel shader, hr %#x.\n", hr);
-        goto err;
-    }
-
     if (FAILED(hr = IDXGISurface_GetDesc(surface, &surface_desc)))
     {
         WARN("Failed to get surface desc, hr %#x.\n", hr);
         goto err;
     }
 
+    render_target->format = desc->pixelFormat;
     render_target->pixel_size.width = surface_desc.Width;
     render_target->pixel_size.height = surface_desc.Height;
     render_target->drawing_state.transform = identity;
@@ -1703,20 +1827,26 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
     return S_OK;
 
 err:
-    if (render_target->rect_bitmap_ps)
-        ID3D10PixelShader_Release(render_target->rect_bitmap_ps);
-    if (render_target->rect_solid_ps)
-        ID3D10PixelShader_Release(render_target->rect_solid_ps);
     if (render_target->bs)
         ID3D10BlendState_Release(render_target->bs);
     if (render_target->rs)
         ID3D10RasterizerState_Release(render_target->rs);
-    if (render_target->vs)
-        ID3D10VertexShader_Release(render_target->vs);
     if (render_target->vb)
         ID3D10Buffer_Release(render_target->vb);
-    if (render_target->il)
-        ID3D10InputLayout_Release(render_target->il);
+    if (render_target->ib)
+        ID3D10Buffer_Release(render_target->ib);
+    for (i = 0; i < D2D_SHAPE_TYPE_COUNT; ++i)
+    {
+        for (j = 0; j < D2D_BRUSH_TYPE_COUNT; ++j)
+        {
+            if (render_target->shape_resources[i].ps[j])
+                ID3D10PixelShader_Release(render_target->shape_resources[i].ps[j]);
+        }
+        if (render_target->shape_resources[i].vs)
+            ID3D10VertexShader_Release(render_target->shape_resources[i].vs);
+        if (render_target->shape_resources[i].il)
+            ID3D10InputLayout_Release(render_target->shape_resources[i].il);
+    }
     if (render_target->stateblock)
         render_target->stateblock->lpVtbl->Release(render_target->stateblock);
     if (render_target->view)
