@@ -1296,9 +1296,17 @@ HRESULT texture_init(struct d3d9_texture *texture, struct d3d9_device *device,
     if (pool != D3DPOOL_DEFAULT || (usage & D3DUSAGE_DYNAMIC))
         surface_flags |= WINED3D_SURFACE_MAPPABLE;
 
+    if (!levels)
+    {
+        if (usage & D3DUSAGE_AUTOGENMIPMAP)
+            levels = 1;
+        else
+            levels = wined3d_log2i(max(width, height)) + 1;
+    }
+
     wined3d_mutex_lock();
     hr = wined3d_texture_create(device->wined3d_device, &desc, levels, surface_flags,
-            texture, &d3d9_texture_wined3d_parent_ops, &texture->wined3d_texture);
+            NULL, texture, &d3d9_texture_wined3d_parent_ops, &texture->wined3d_texture);
     wined3d_mutex_unlock();
     if (FAILED(hr))
     {
@@ -1338,9 +1346,17 @@ HRESULT cubetexture_init(struct d3d9_texture *texture, struct d3d9_device *devic
     if (pool != D3DPOOL_DEFAULT || (usage & D3DUSAGE_DYNAMIC))
         surface_flags |= WINED3D_SURFACE_MAPPABLE;
 
+    if (!levels)
+    {
+        if (usage & D3DUSAGE_AUTOGENMIPMAP)
+            levels = 1;
+        else
+            levels = wined3d_log2i(edge_length) + 1;
+    }
+
     wined3d_mutex_lock();
     hr = wined3d_texture_create(device->wined3d_device, &desc, levels, surface_flags,
-            texture, &d3d9_texture_wined3d_parent_ops, &texture->wined3d_texture);
+            NULL, texture, &d3d9_texture_wined3d_parent_ops, &texture->wined3d_texture);
     wined3d_mutex_unlock();
     if (FAILED(hr))
     {
@@ -1376,9 +1392,17 @@ HRESULT volumetexture_init(struct d3d9_texture *texture, struct d3d9_device *dev
     desc.depth = depth;
     desc.size = 0;
 
+    if (!levels)
+    {
+        if (usage & D3DUSAGE_AUTOGENMIPMAP)
+            levels = 1;
+        else
+            levels = wined3d_log2i(max(max(width, height), depth)) + 1;
+    }
+
     wined3d_mutex_lock();
     hr = wined3d_texture_create(device->wined3d_device, &desc, levels, 0,
-            texture, &d3d9_texture_wined3d_parent_ops, &texture->wined3d_texture);
+            NULL, texture, &d3d9_texture_wined3d_parent_ops, &texture->wined3d_texture);
     wined3d_mutex_unlock();
     if (FAILED(hr))
     {
