@@ -51,8 +51,11 @@
 #include "ipstats.h"
 #include "ipifcons.h"
 #include "fltdefs.h"
+#include "ifdef.h"
+#include "netioapi.h"
 
 #include "wine/debug.h"
+#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(iphlpapi);
 
@@ -198,6 +201,15 @@ BOOL WINAPI CancelIPChangeNotify(LPOVERLAPPED overlapped)
 }
 
 
+/******************************************************************
+ *    CancelMibChangeNotify2 (IPHLPAPI.@)
+ */
+DWORD WINAPI CancelMibChangeNotify2(HANDLE handle)
+{
+    FIXME("(handle %p): stub\n", handle);
+    return NO_ERROR;
+}
+
 
 /******************************************************************
  *    CreateIpForwardEntry (IPHLPAPI.@)
@@ -269,6 +281,142 @@ DWORD WINAPI CreateProxyArpEntry(DWORD dwAddress, DWORD dwMask, DWORD dwIfIndex)
   return ERROR_NOT_SUPPORTED;
 }
 
+static char *debugstr_ipv6(const struct WS_sockaddr_in6 *sin, char *buf)
+{
+    const IN6_ADDR *addr = &sin->sin6_addr;
+    char *p = buf;
+    int i;
+    BOOL in_zero = FALSE;
+
+    for (i = 0; i < 7; i++)
+    {
+        if (!addr->u.Word[i])
+        {
+            if (i == 0)
+                *p++ = ':';
+            if (!in_zero)
+            {
+                *p++ = ':';
+                in_zero = TRUE;
+            }
+        }
+        else
+        {
+            p += sprintf(p, "%x:", ntohs(addr->u.Word[i]));
+            in_zero = FALSE;
+        }
+    }
+    sprintf(p, "%x", ntohs(addr->u.Word[7]));
+    return buf;
+}
+
+static BOOL map_address_6to4( SOCKADDR_IN6 *addr6, SOCKADDR_IN *addr4 )
+{
+    ULONG i;
+
+    if (addr6->sin6_family != WS_AF_INET6) return FALSE;
+
+    for (i = 0; i < 5; i++)
+        if (addr6->sin6_addr.u.Word[i]) return FALSE;
+
+    if (addr6->sin6_addr.u.Word[5] != 0xffff) return FALSE;
+
+    addr4->sin_family = AF_INET;
+    addr4->sin_port   = addr6->sin6_port;
+    addr4->sin_addr.S_un.S_addr = addr6->sin6_addr.u.Word[6] << 16 | addr6->sin6_addr.u.Word[7];
+    memset( &addr4->sin_zero, 0, sizeof(addr4->sin_zero) );
+
+    return TRUE;
+}
+
+static BOOL find_src_address( MIB_IPADDRTABLE *table, SOCKADDR_IN *dst, SOCKADDR_IN6 *src )
+{
+    MIB_IPFORWARDROW row;
+    DWORD i, j;
+
+    if (GetBestRoute( dst->sin_addr.S_un.S_addr, 0, &row )) return FALSE;
+
+    for (i = 0; i < table->dwNumEntries; i++)
+    {
+        /* take the first address */
+        if (table->table[i].dwIndex == row.dwForwardIfIndex)
+        {
+            src->sin6_family   = WS_AF_INET6;
+            src->sin6_port     = 0;
+            src->sin6_flowinfo = 0;
+            for (j = 0; j < 5; j++) src->sin6_addr.u.Word[j] = 0;
+            src->sin6_addr.u.Word[5] = 0xffff;
+            src->sin6_addr.u.Word[6] = table->table[i].dwAddr & 0xffff;
+            src->sin6_addr.u.Word[7] = table->table[i].dwAddr >> 16;
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+/******************************************************************
+ *    CreateSortedAddressPairs (IPHLPAPI.@)
+ */
+DWORD WINAPI CreateSortedAddressPairs( const PSOCKADDR_IN6 src_list, DWORD src_count,
+                                       const PSOCKADDR_IN6 dst_list, DWORD dst_count,
+                                       DWORD options, PSOCKADDR_IN6_PAIR *pair_list,
+                                       DWORD *pair_count )
+{
+    DWORD i, size, ret;
+    SOCKADDR_IN6_PAIR *pairs;
+    SOCKADDR_IN6 *ptr;
+    SOCKADDR_IN addr4;
+    MIB_IPADDRTABLE *table;
+
+    FIXME( "(src_list %p src_count %u dst_list %p dst_count %u options %x pair_list %p pair_count %p): stub\n",
+           src_list, src_count, dst_list, dst_count, options, pair_list, pair_count );
+
+    if (src_list || src_count || !dst_list || !pair_list || !pair_count || dst_count > 500)
+        return ERROR_INVALID_PARAMETER;
+
+    for (i = 0; i < dst_count; i++)
+    {
+        if (!map_address_6to4( &dst_list[i], &addr4 ))
+        {
+            FIXME("only mapped IPv4 addresses are supported\n");
+            return ERROR_NOT_SUPPORTED;
+        }
+    }
+
+    size = dst_count * sizeof(*pairs);
+    size += dst_count * sizeof(SOCKADDR_IN6) * 2; /* source address + destination address */
+    if (!(pairs = HeapAlloc( GetProcessHeap(), 0, size ))) return ERROR_NOT_ENOUGH_MEMORY;
+    ptr = (SOCKADDR_IN6 *)(char *)pairs + dst_count * sizeof(*pairs);
+
+    if ((ret = getIPAddrTable( &table, GetProcessHeap(), 0 )))
+    {
+        HeapFree( GetProcessHeap(), 0, pairs );
+        return ret;
+    }
+
+    for (i = 0; i < dst_count; i++)
+    {
+        pairs[i].SourceAddress = ptr++;
+        if (!map_address_6to4( &dst_list[i], &addr4 ) ||
+            !find_src_address( table, &addr4, pairs[i].SourceAddress ))
+        {
+            char buf[46];
+            FIXME( "source address for %s not found\n", debugstr_ipv6(&dst_list[i], buf) );
+            memset( pairs[i].SourceAddress, 0, sizeof(*pairs[i].SourceAddress) );
+            pairs[i].SourceAddress->sin6_family = WS_AF_INET6;
+        }
+
+        pairs[i].DestinationAddress = ptr++;
+        memcpy( pairs[i].DestinationAddress, &dst_list[i], sizeof(*pairs[i].DestinationAddress) );
+    }
+    *pair_list = pairs;
+    *pair_count = dst_count;
+
+    HeapFree( GetProcessHeap(), 0, table );
+    return NO_ERROR;
+}
+
 
 /******************************************************************
  *    DeleteIPAddress (IPHLPAPI.@)
@@ -411,6 +559,20 @@ DWORD WINAPI FlushIpNetTable(DWORD dwIfIndex)
   return ERROR_NOT_SUPPORTED;
 }
 
+/******************************************************************
+ *    FreeMibTable (IPHLPAPI.@)
+ *
+ * Free buffer allocated by network functions
+ *
+ * PARAMS
+ *  ptr     [In] pointer to the buffer to free
+ *
+ */
+void WINAPI FreeMibTable(void *ptr)
+{
+  TRACE("(%p)\n", ptr);
+  HeapFree(GetProcessHeap(), 0, ptr);
+}
 
 /******************************************************************
  *    GetAdapterIndex (IPHLPAPI.@)
@@ -559,6 +721,12 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
                   }
                 }
               }
+              /* If no IP was found it probably means that the interface is not
+               * configured. In this case we have to return a zeroed IP and mask. */
+              if (firstIPAddr) {
+                strcpy(ptr->IpAddressList.IpAddress.String, "0.0.0.0");
+                strcpy(ptr->IpAddressList.IpMask.String, "0.0.0.0");
+              }
               /* Find first router through this interface, which we'll assume
                * is the default gateway for this adapter */
               for (i = 0; i < routeTable->dwNumEntries; i++)
@@ -676,35 +844,6 @@ static char *debugstr_ipv4(const in_addr_t *in_addr, char *buf)
     return buf;
 }
 
-static char *debugstr_ipv6(const struct WS_sockaddr_in6 *sin, char *buf)
-{
-    const IN6_ADDR *addr = &sin->sin6_addr;
-    char *p = buf;
-    int i;
-    BOOL in_zero = FALSE;
-
-    for (i = 0; i < 7; i++)
-    {
-        if (!addr->u.Word[i])
-        {
-            if (i == 0)
-                *p++ = ':';
-            if (!in_zero)
-            {
-                *p++ = ':';
-                in_zero = TRUE;
-            }
-        }
-        else
-        {
-            p += sprintf(p, "%x:", ntohs(addr->u.Word[i]));
-            in_zero = FALSE;
-        }
-    }
-    sprintf(p, "%x", ntohs(addr->u.Word[7]));
-    return buf;
-}
-
 static ULONG count_v4_gateways(DWORD index, PMIB_IPFORWARDTABLE routeTable)
 {
     DWORD i, num_gateways = 0;
@@ -733,6 +872,31 @@ static PMIB_IPFORWARDROW findIPv4Gateway(DWORD index,
     return row;
 }
 
+static void fill_unicast_addr_data(IP_ADAPTER_ADDRESSES *aa, IP_ADAPTER_UNICAST_ADDRESS *ua)
+{
+    /* Actually this information should be read somewhere from the system
+     * but it doesn't matter much for the bugs found so far.
+     * This information is required for DirectPlay8 games. */
+    if (aa->IfType != IF_TYPE_SOFTWARE_LOOPBACK)
+    {
+        ua->PrefixOrigin = IpPrefixOriginDhcp;
+        ua->SuffixOrigin = IpSuffixOriginDhcp;
+    }
+    else
+    {
+        ua->PrefixOrigin = IpPrefixOriginManual;
+        ua->SuffixOrigin = IpSuffixOriginManual;
+    }
+
+    /* The address is not duplicated in the network */
+    ua->DadState = IpDadStatePreferred;
+
+    /* Some address life time values, required even for non-dhcp addresses */
+    ua->ValidLifetime = 60000;
+    ua->PreferredLifetime = 60000;
+    ua->LeaseLifetime = 60000;
+}
+
 static ULONG adapterAddressesFromIndex(ULONG family, ULONG flags, IF_INDEX index,
                                        IP_ADAPTER_ADDRESSES *aa, ULONG *size)
 {
@@ -833,6 +997,13 @@ static ULONG adapterAddressesFromIndex(ULONG family, ULONG flags, IF_INDEX index
 
         TRACE("%s: %d IPv4 addresses, %d IPv6 addresses:\n", name, num_v4addrs,
               num_v6addrs);
+
+        buflen = MAX_INTERFACE_PHYSADDR;
+        getInterfacePhysicalByIndex(index, &buflen, aa->PhysicalAddress, &type);
+        aa->PhysicalAddressLength = buflen;
+        aa->IfType = typeFromMibType(type);
+        aa->ConnectionType = connectionTypeFromMibType(type);
+
         if (num_v4_gateways)
         {
             PMIB_IPFORWARDROW adapterRow;
@@ -879,6 +1050,7 @@ static ULONG adapterAddressesFromIndex(ULONG family, ULONG flags, IF_INDEX index
                 sa->sin_port             = 0;
                 TRACE("IPv4 %d/%d: %s\n", i + 1, num_v4addrs,
                       debugstr_ipv4(&sa->sin_addr.S_un.S_addr, addr_buf));
+                fill_unicast_addr_data(aa, ua);
 
                 ptr += ua->u.s.Length + ua->Address.iSockaddrLength;
                 if (i < num_v4addrs - 1)
@@ -916,6 +1088,7 @@ static ULONG adapterAddressesFromIndex(ULONG family, ULONG flags, IF_INDEX index
                 memcpy(sa, v6addrs[i].lpSockaddr, sizeof(*sa));
                 TRACE("IPv6 %d/%d: %s\n", i + 1, num_v6addrs,
                       debugstr_ipv6(sa, addr_buf));
+                fill_unicast_addr_data(aa, ua);
 
                 ptr += ua->u.s.Length + ua->Address.iSockaddrLength;
                 if (i < num_v6addrs - 1)
@@ -983,6 +1156,7 @@ static ULONG adapterAddressesFromIndex(ULONG family, ULONG flags, IF_INDEX index
                 struct WS_sockaddr_in6 *sa;
                 const IN6_ADDR *addr, *mask;
                 BOOL done = FALSE;
+                ULONG k;
 
                 prefix->u.s.Length = sizeof(*prefix);
                 prefix->u.s.Flags  = 0;
@@ -1000,11 +1174,11 @@ static ULONG adapterAddressesFromIndex(ULONG family, ULONG flags, IF_INDEX index
                 sa->sin6_scope_id = 0;
 
                 prefix->PrefixLength = 0;
-                for (i = 0; i < 8 && !done; i++)
+                for (k = 0; k < 8 && !done; k++)
                 {
                     for (j = 0; j < sizeof(WORD) * 8 && !done; j++)
                     {
-                        if (mask->u.Word[i] & 1 << j) prefix->PrefixLength++;
+                        if (mask->u.Word[k] & 1 << j) prefix->PrefixLength++;
                         else done = TRUE;
                     }
                 }
@@ -1019,12 +1193,6 @@ static ULONG adapterAddressesFromIndex(ULONG family, ULONG flags, IF_INDEX index
             }
         }
 
-        buflen = MAX_INTERFACE_PHYSADDR;
-        getInterfacePhysicalByIndex(index, &buflen, aa->PhysicalAddress, &type);
-        aa->PhysicalAddressLength = buflen;
-        aa->IfType = typeFromMibType(type);
-        aa->ConnectionType = connectionTypeFromMibType(type);
-
         getInterfaceMtuByName(name, &aa->Mtu);
 
         getInterfaceStatusByName(name, &status);
@@ -1250,7 +1418,7 @@ ULONG WINAPI DECLSPEC_HOTPATCH GetAdaptersAddresses(ULONG family, ULONG flags, P
                                                     PIP_ADAPTER_ADDRESSES aa, PULONG buflen)
 {
     InterfaceIndexTable *table;
-    ULONG i, size, dns_server_size, dns_suffix_size, total_size, ret = ERROR_NO_DATA;
+    ULONG i, size, dns_server_size = 0, dns_suffix_size, total_size, ret = ERROR_NO_DATA;
 
     TRACE("(%d, %08x, %p, %p, %p)\n", family, flags, reserved, aa, buflen);
 
@@ -1307,7 +1475,7 @@ ULONG WINAPI DECLSPEC_HOTPATCH GetAdaptersAddresses(ULONG family, ULONG flags, P
                 size = bytes_left -= size;
             }
         }
-        if (!(flags & GAA_FLAG_SKIP_DNS_SERVER) && dns_server_size)
+        if (dns_server_size)
         {
             firstDns = (PIP_ADAPTER_DNS_SERVER_ADDRESS)((BYTE *)first_aa + total_size - dns_server_size - dns_suffix_size);
             get_dns_server_addresses(firstDns, &dns_server_size);
@@ -1330,8 +1498,10 @@ ULONG WINAPI DECLSPEC_HOTPATCH GetAdaptersAddresses(ULONG family, ULONG flags, P
         ret = ERROR_SUCCESS;
     }
     else
+    {
         ret = ERROR_BUFFER_OVERFLOW;
-    *buflen = total_size;
+        *buflen = total_size;
+    }
 
     TRACE("num adapters %u\n", table->numIndexes);
     HeapFree(GetProcessHeap(), 0, table);
@@ -1516,6 +1686,57 @@ DWORD WINAPI GetIfEntry(PMIB_IFROW pIfRow)
   return ret;
 }
 
+/******************************************************************
+ *    GetIfEntry2 (IPHLPAPI.@)
+ */
+DWORD WINAPI GetIfEntry2( MIB_IF_ROW2 *row2 )
+{
+    DWORD ret, len = sizeof(row2->Description)/sizeof(row2->Description[0]);
+    char buf[MAX_ADAPTER_NAME], *name;
+    MIB_IFROW row;
+
+    TRACE("%p\n", row2);
+
+    if (!row2 || (!(name = getInterfaceNameByIndex( row2->InterfaceIndex, buf )) &&
+                  !(name = getInterfaceNameByIndex( row2->InterfaceLuid.Info.NetLuidIndex, buf ))))
+    {
+        return ERROR_INVALID_PARAMETER;
+    }
+    if ((ret = getInterfaceEntryByName( name, &row ))) return ret;
+    if ((ret = getInterfaceStatsByName( name, &row ))) return ret;
+
+    memset( row2, 0, sizeof(*row2) );
+    row2->InterfaceLuid.Info.Reserved     = 0;
+    row2->InterfaceLuid.Info.NetLuidIndex = row.dwIndex;
+    row2->InterfaceLuid.Info.IfType       = row.dwType;
+    row2->InterfaceIndex                  = row.dwIndex;
+    row2->InterfaceGuid.Data1             = row.dwIndex;
+    row2->Type                            = row.dwType;
+    row2->Mtu                             = row.dwMtu;
+    MultiByteToWideChar( CP_UNIXCP, 0, (const char *)row.bDescr, -1, row2->Description, len );
+    row2->PhysicalAddressLength           = row.dwPhysAddrLen;
+    memcpy( &row2->PhysicalAddress, &row.bPhysAddr, row.dwPhysAddrLen );
+    memcpy( &row2->PermanentPhysicalAddress, &row.bPhysAddr, row.dwPhysAddrLen );
+    row2->OperStatus                      = IfOperStatusUp;
+    row2->AdminStatus                     = NET_IF_ADMIN_STATUS_UP;
+    row2->MediaConnectState               = MediaConnectStateConnected;
+    row2->ConnectionType                  = NET_IF_CONNECTION_DEDICATED;
+
+    /* stats */
+    row2->InOctets        = row.dwInOctets;
+    row2->InUcastPkts     = row.dwInUcastPkts;
+    row2->InNUcastPkts    = row.dwInNUcastPkts;
+    row2->InDiscards      = row.dwInDiscards;
+    row2->InErrors        = row.dwInErrors;
+    row2->InUnknownProtos = row.dwInUnknownProtos;
+    row2->OutOctets       = row.dwOutOctets;
+    row2->OutUcastPkts    = row.dwOutUcastPkts;
+    row2->OutNUcastPkts   = row.dwOutNUcastPkts;
+    row2->OutDiscards     = row.dwOutDiscards;
+    row2->OutErrors       = row.dwOutErrors;
+
+    return NO_ERROR;
+}
 
 static int IfTableSorter(const void *a, const void *b)
 {
@@ -2282,6 +2503,19 @@ DWORD WINAPI NotifyAddrChange(PHANDLE Handle, LPOVERLAPPED overlapped)
 }
 
 
+/******************************************************************
+ *    NotifyIpInterfaceChange (IPHLPAPI.@)
+ */
+DWORD WINAPI NotifyIpInterfaceChange(ULONG family, PVOID callback, PVOID context,
+                                     BOOLEAN init_notify, PHANDLE handle)
+{
+    FIXME("(family %d, callback %p, context %p, init_notify %d, handle %p): stub\n",
+          family, callback, context, init_notify, handle);
+    if (handle) *handle = NULL;
+    return ERROR_NOT_SUPPORTED;
+}
+
+
 /******************************************************************
  *    NotifyRouteChange (IPHLPAPI.@)
  *
@@ -2563,3 +2797,177 @@ ULONG WINAPI GetTcp6Table2(PMIB_TCP6TABLE2 table, PULONG size, BOOL order)
     FIXME("pTcp6Table2 %p, size %p, order %d: stub\n", table, size, order);
     return ERROR_NOT_SUPPORTED;
 }
+
+/******************************************************************
+ *    ConvertInterfaceGuidToLuid (IPHLPAPI.@)
+ */
+DWORD WINAPI ConvertInterfaceGuidToLuid(const GUID *guid, NET_LUID *luid)
+{
+    DWORD ret;
+    MIB_IFROW row;
+
+    TRACE("(%s %p)\n", debugstr_guid(guid), luid);
+
+    if (!guid || !luid) return ERROR_INVALID_PARAMETER;
+
+    row.dwIndex = guid->Data1;
+    if ((ret = GetIfEntry( &row ))) return ret;
+
+    luid->Info.Reserved     = 0;
+    luid->Info.NetLuidIndex = guid->Data1;
+    luid->Info.IfType       = row.dwType;
+    return NO_ERROR;
+}
+
+/******************************************************************
+ *    ConvertInterfaceIndexToLuid (IPHLPAPI.@)
+ */
+DWORD WINAPI ConvertInterfaceIndexToLuid(NET_IFINDEX index, NET_LUID *luid)
+{
+    MIB_IFROW row;
+
+    TRACE("(%u %p)\n", index, luid);
+
+    if (!luid) return ERROR_INVALID_PARAMETER;
+    memset( luid, 0, sizeof(*luid) );
+
+    row.dwIndex = index;
+    if (GetIfEntry( &row )) return ERROR_FILE_NOT_FOUND;
+
+    luid->Info.Reserved     = 0;
+    luid->Info.NetLuidIndex = index;
+    luid->Info.IfType       = row.dwType;
+    return NO_ERROR;
+}
+
+/******************************************************************
+ *    ConvertInterfaceLuidToGuid (IPHLPAPI.@)
+ */
+DWORD WINAPI ConvertInterfaceLuidToGuid(const NET_LUID *luid, GUID *guid)
+{
+    DWORD ret;
+    MIB_IFROW row;
+
+    TRACE("(%p %p)\n", luid, guid);
+
+    if (!luid || !guid) return ERROR_INVALID_PARAMETER;
+
+    row.dwIndex = luid->Info.NetLuidIndex;
+    if ((ret = GetIfEntry( &row ))) return ret;
+
+    guid->Data1 = luid->Info.NetLuidIndex;
+    return NO_ERROR;
+}
+
+/******************************************************************
+ *    ConvertInterfaceLuidToIndex (IPHLPAPI.@)
+ */
+DWORD WINAPI ConvertInterfaceLuidToIndex(const NET_LUID *luid, NET_IFINDEX *index)
+{
+    DWORD ret;
+    MIB_IFROW row;
+
+    TRACE("(%p %p)\n", luid, index);
+
+    if (!luid || !index) return ERROR_INVALID_PARAMETER;
+
+    row.dwIndex = luid->Info.NetLuidIndex;
+    if ((ret = GetIfEntry( &row ))) return ret;
+
+    *index = luid->Info.NetLuidIndex;
+    return NO_ERROR;
+}
+
+/******************************************************************
+ *    ConvertInterfaceLuidToNameA (IPHLPAPI.@)
+ */
+DWORD WINAPI ConvertInterfaceLuidToNameA(const NET_LUID *luid, char *name, SIZE_T len)
+{
+    DWORD ret;
+    MIB_IFROW row;
+
+    TRACE("(%p %p %u)\n", luid, name, (DWORD)len);
+
+    if (!luid) return ERROR_INVALID_PARAMETER;
+
+    row.dwIndex = luid->Info.NetLuidIndex;
+    if ((ret = GetIfEntry( &row ))) return ret;
+
+    if (!name || len < WideCharToMultiByte( CP_UNIXCP, 0, row.wszName, -1, NULL, 0, NULL, NULL ))
+        return ERROR_NOT_ENOUGH_MEMORY;
+
+    WideCharToMultiByte( CP_UNIXCP, 0, row.wszName, -1, name, len, NULL, NULL );
+    return NO_ERROR;
+}
+
+/******************************************************************
+ *    ConvertInterfaceLuidToNameW (IPHLPAPI.@)
+ */
+DWORD WINAPI ConvertInterfaceLuidToNameW(const NET_LUID *luid, WCHAR *name, SIZE_T len)
+{
+    DWORD ret;
+    MIB_IFROW row;
+
+    TRACE("(%p %p %u)\n", luid, name, (DWORD)len);
+
+    if (!luid || !name) return ERROR_INVALID_PARAMETER;
+
+    row.dwIndex = luid->Info.NetLuidIndex;
+    if ((ret = GetIfEntry( &row ))) return ret;
+
+    if (len < strlenW( row.wszName ) + 1) return ERROR_NOT_ENOUGH_MEMORY;
+    strcpyW( name, row.wszName );
+    return NO_ERROR;
+}
+
+/******************************************************************
+ *    ConvertInterfaceNameToLuidA (IPHLPAPI.@)
+ */
+DWORD WINAPI ConvertInterfaceNameToLuidA(const char *name, NET_LUID *luid)
+{
+    DWORD ret;
+    IF_INDEX index;
+    MIB_IFROW row;
+
+    TRACE("(%s %p)\n", debugstr_a(name), luid);
+
+    if ((ret = getInterfaceIndexByName( name, &index ))) return ERROR_INVALID_NAME;
+    if (!luid) return ERROR_INVALID_PARAMETER;
+
+    row.dwIndex = index;
+    if ((ret = GetIfEntry( &row ))) return ret;
+
+    luid->Info.Reserved     = 0;
+    luid->Info.NetLuidIndex = index;
+    luid->Info.IfType       = row.dwType;
+    return NO_ERROR;
+}
+
+/******************************************************************
+ *    ConvertInterfaceNameToLuidW (IPHLPAPI.@)
+ */
+DWORD WINAPI ConvertInterfaceNameToLuidW(const WCHAR *name, NET_LUID *luid)
+{
+    DWORD ret;
+    IF_INDEX index;
+    MIB_IFROW row;
+    char nameA[IF_MAX_STRING_SIZE + 1];
+
+    TRACE("(%s %p)\n", debugstr_w(name), luid);
+
+    if (!luid) return ERROR_INVALID_PARAMETER;
+    memset( luid, 0, sizeof(*luid) );
+
+    if (!WideCharToMultiByte( CP_UNIXCP, 0, name, -1, nameA, sizeof(nameA), NULL, NULL ))
+        return ERROR_INVALID_NAME;
+
+    if ((ret = getInterfaceIndexByName( nameA, &index ))) return ret;
+
+    row.dwIndex = index;
+    if ((ret = GetIfEntry( &row ))) return ret;
+
+    luid->Info.Reserved     = 0;
+    luid->Info.NetLuidIndex = index;
+    luid->Info.IfType       = row.dwType;
+    return NO_ERROR;
+}
