@@ -395,6 +395,7 @@ static void create_xmlhttprequest(IHTMLDocument2 *doc)
         return;
     }
 
+    VariantInit(&var);
     hres = IHTMLWindow5_get_XMLHttpRequest(window5, &var);
     IHTMLWindow5_Release(window5);
     ok(hres == S_OK, "get_XMLHttpRequest failed: %08x\n", hres);
@@ -448,12 +449,12 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
     ok(val == 0, "Expect UNSENT, got %d\n", val);
 
     hres = IHTMLXMLHttpRequest_get_status(xhr, NULL);
-    todo_wine ok(hres == E_POINTER, "Expect E_POINTER, got %08x\n", hres);
+    ok(hres == E_POINTER, "Expect E_POINTER, got %08x\n", hres);
 
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
-    todo_wine ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
-    todo_wine ok(val == 0, "Expect 0, got %d\n", val);
+    ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
+    ok(val == 0, "Expect 0, got %d\n", val);
 
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, NULL);
     todo_wine ok(hres == E_POINTER, "Expect E_POINTER, got %08x\n", hres);
@@ -560,8 +561,8 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
 
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
-    todo_wine ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
-    todo_wine ok(val == 0, "Expect 0, got %d\n", val);
+    ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
+    ok(val == 0, "Expect 0, got %d\n", val);
 
     text = (BSTR)0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, &text);
@@ -595,8 +596,8 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
 
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
-    todo_wine ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
-    todo_wine ok(val == 0, "Expect 0, got %d\n", val);
+    ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
+    ok(val == 0, "Expect 0, got %d\n", val);
 
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, &text);
     todo_wine ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
@@ -629,8 +630,8 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
 
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
-    todo_wine ok(hres == S_OK, "get_status failed: %08x\n", hres);
-    todo_wine ok(val == 200, "Expect 200, got %d\n", val);
+    ok(hres == S_OK, "get_status failed: %08x\n", hres);
+    ok(val == 200, "Expect 200, got %d\n", val);
 
     text = NULL;
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, &text);
@@ -646,9 +647,9 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
 
     text = NULL;
     hres = IHTMLXMLHttpRequest_get_responseText(xhr, &text);
-    todo_wine ok(hres == S_OK, "get_responseText failed: %08x\n", hres);
-    todo_wine ok(text != NULL, "test == NULL\n");
-    todo_wine ok(!strcmp_wa(text, EXPECT_RESPONSE_TEXT), "expect %s, got %s\n",
+    ok(hres == S_OK, "get_responseText failed: %08x\n", hres);
+    ok(text != NULL, "test == NULL\n");
+    ok(!strcmp_wa(text, EXPECT_RESPONSE_TEXT), "expect %s, got %s\n",
         EXPECT_RESPONSE_TEXT, wine_dbgstr_w(text));
     SysFreeString(text);
 
