@@ -1377,6 +1377,16 @@ HRESULT WINAPI SetCurrentProcessExplicitAppUserModelID(PCWSTR appid)
     return E_NOTIMPL;
 }
 
+/***********************************************************************
+ *              GetCurrentProcessExplicitAppUserModelID (SHELL32.@)
+ */
+HRESULT WINAPI GetCurrentProcessExplicitAppUserModelID(PWSTR *appid)
+{
+    FIXME("%p: stub\n", appid);
+    *appid = NULL;
+    return E_NOTIMPL;
+}
+
 /***********************************************************************
  *              SHSetUnreadMailCountW (SHELL32.@)
  */
