@@ -158,9 +158,9 @@ static void setup_test( struct info *info, enum api function, unsigned int line
 
 static void test_connection_cache( void )
 {
-    HANDLE ses, con, req;
+    HANDLE ses, con, req, event;
     DWORD size, status;
-    BOOL ret;
+    BOOL ret, unload = TRUE;
     struct info info, *context = &info;
 
     info.test  = cache_test;
@@ -171,6 +171,14 @@ static void test_connection_cache( void )
     ses = WinHttpOpen( user_agent, 0, NULL, NULL, 0 );
     ok(ses != NULL, "failed to open session %u\n", GetLastError());
 
+    event = CreateEventW( NULL, FALSE, FALSE, NULL );
+    ret = WinHttpSetOption( ses, WINHTTP_OPTION_UNLOAD_NOTIFY_EVENT, &event, sizeof(event) );
+    if (!ret)
+    {
+        win_skip("Unload event not supported\n");
+        unload = FALSE;
+    }
+
     WinHttpSetStatusCallback( ses, check_notification, WINHTTP_CALLBACK_FLAG_ALL_NOTIFICATIONS, 0 );
 
     ret = WinHttpSetOption( ses, WINHTTP_OPTION_CONTEXT_VALUE, &context, sizeof(struct info *) );
@@ -186,6 +194,11 @@ static void test_connection_cache( void )
 
     setup_test( &info, winhttp_send_request, __LINE__ );
     ret = WinHttpSendRequest( req, NULL, 0, NULL, 0, 0, 0 );
+    if (!ret && GetLastError() == ERROR_WINHTTP_CANNOT_CONNECT)
+    {
+        skip("connection failed, skipping\n");
+        goto done;
+    }
     ok(ret, "failed to send request %u\n", GetLastError());
 
     setup_test( &info, winhttp_receive_response, __LINE__ );
@@ -209,6 +222,11 @@ static void test_connection_cache( void )
 
     setup_test( &info, winhttp_send_request, __LINE__ );
     ret = WinHttpSendRequest( req, NULL, 0, NULL, 0, 0, 0 );
+    if (!ret && GetLastError() == ERROR_WINHTTP_CANNOT_CONNECT)
+    {
+        skip("connection failed, skipping\n");
+        goto done;
+    }
     ok(ret, "failed to send request %u\n", GetLastError());
 
     setup_test( &info, winhttp_receive_response, __LINE__ );
@@ -226,15 +244,33 @@ static void test_connection_cache( void )
     setup_test( &info, winhttp_close_handle, __LINE__ );
     WinHttpCloseHandle( req );
     WinHttpCloseHandle( con );
+
+    if (unload)
+    {
+        status = WaitForSingleObject( event, 0 );
+        ok(status == WAIT_TIMEOUT, "got %08x\n", status);
+    }
+
     WinHttpCloseHandle( ses );
 
     Sleep(2000); /* make sure connection is evicted from cache */
+    if (unload)
+    {
+        status = WaitForSingleObject( event, 0 );
+        ok(status == WAIT_OBJECT_0, "got %08x\n", status);
+    }
 
     info.index = 0;
 
     ses = WinHttpOpen( user_agent, 0, NULL, NULL, 0 );
     ok(ses != NULL, "failed to open session %u\n", GetLastError());
 
+    if (unload)
+    {
+        ret = WinHttpSetOption( ses, WINHTTP_OPTION_UNLOAD_NOTIFY_EVENT, &event, sizeof(event) );
+        ok(ret, "failed to set unload option\n");
+    }
+
     WinHttpSetStatusCallback( ses, check_notification, WINHTTP_CALLBACK_FLAG_ALL_NOTIFICATIONS, 0 );
 
     ret = WinHttpSetOption( ses, WINHTTP_OPTION_CONTEXT_VALUE, &context, sizeof(struct info *) );
@@ -253,6 +289,11 @@ static void test_connection_cache( void )
 
     setup_test( &info, winhttp_send_request, __LINE__ );
     ret = WinHttpSendRequest( req, NULL, 0, NULL, 0, 0, 0 );
+    if (!ret && GetLastError() == ERROR_WINHTTP_CANNOT_CONNECT)
+    {
+        skip("connection failed, skipping\n");
+        goto done;
+    }
     ok(ret, "failed to send request %u\n", GetLastError());
 
     setup_test( &info, winhttp_receive_response, __LINE__ );
@@ -276,6 +317,11 @@ static void test_connection_cache( void )
 
     setup_test( &info, winhttp_send_request, __LINE__ );
     ret = WinHttpSendRequest( req, NULL, 0, NULL, 0, 0, 0 );
+    if (!ret && GetLastError() == ERROR_WINHTTP_CANNOT_CONNECT)
+    {
+        skip("connection failed, skipping\n");
+        goto done;
+    }
     ok(ret, "failed to send request %u\n", GetLastError());
 
     setup_test( &info, winhttp_receive_response, __LINE__ );
@@ -288,11 +334,26 @@ static void test_connection_cache( void )
     ok(status == 200, "request failed unexpectedly %u\n", status);
 
     setup_test( &info, winhttp_close_handle, __LINE__ );
+done:
     WinHttpCloseHandle( req );
     WinHttpCloseHandle( con );
+
+    if (unload)
+    {
+        status = WaitForSingleObject( event, 0 );
+        ok(status == WAIT_TIMEOUT, "got %08x\n", status);
+    }
+
     WinHttpCloseHandle( ses );
 
     Sleep(2000); /* make sure connection is evicted from cache */
+    if (unload)
+    {
+        status = WaitForSingleObject( event, 0 );
+        ok(status == WAIT_OBJECT_0, "got %08x\n", status);
+    }
+
+    CloseHandle( event );
 }
 
 static const struct notification redirect_test[] =
@@ -353,6 +414,11 @@ static void test_redirect( void )
 
     setup_test( &info, winhttp_send_request, __LINE__ );
     ret = WinHttpSendRequest( req, NULL, 0, NULL, 0, 0, 0 );
+    if (!ret && GetLastError() == ERROR_WINHTTP_CANNOT_CONNECT)
+    {
+        skip("connection failed, skipping\n");
+        goto done;
+    }
     ok(ret, "failed to send request %u\n", GetLastError());
 
     setup_test( &info, winhttp_receive_response, __LINE__ );
@@ -365,6 +431,7 @@ static void test_redirect( void )
     ok(status == 200, "request failed unexpectedly %u\n", status);
 
     setup_test( &info, winhttp_close_handle, __LINE__ );
+done:
     WinHttpCloseHandle( req );
     WinHttpCloseHandle( con );
     WinHttpCloseHandle( ses );
@@ -406,9 +473,9 @@ static const struct notification async_test[] =
 
 static void test_async( void )
 {
-    HANDLE ses, con, req;
+    HANDLE ses, con, req, event;
     DWORD size, status;
-    BOOL ret;
+    BOOL ret, unload = TRUE;
     struct info info, *context = &info;
     char buffer[1024];
 
@@ -420,6 +487,14 @@ static void test_async( void )
     ses = WinHttpOpen( user_agent, 0, NULL, NULL, WINHTTP_FLAG_ASYNC );
     ok(ses != NULL, "failed to open session %u\n", GetLastError());
 
+    event = CreateEventW( NULL, FALSE, FALSE, NULL );
+    ret = WinHttpSetOption( ses, WINHTTP_OPTION_UNLOAD_NOTIFY_EVENT, &event, sizeof(event) );
+    if (!ret)
+    {
+        win_skip("Unload event not supported\n");
+        unload = FALSE;
+    }
+
     WinHttpSetStatusCallback( ses, check_notification, WINHTTP_CALLBACK_FLAG_ALL_NOTIFICATIONS, 0 );
 
     ret = WinHttpSetOption( ses, WINHTTP_OPTION_CONTEXT_VALUE, &context, sizeof(struct info *) );
@@ -435,6 +510,15 @@ static void test_async( void )
 
     setup_test( &info, winhttp_send_request, __LINE__ );
     ret = WinHttpSendRequest( req, NULL, 0, NULL, 0, 0, 0 );
+    if (!ret && GetLastError() == ERROR_WINHTTP_CANNOT_CONNECT)
+    {
+        skip("connection failed, skipping\n");
+        WinHttpCloseHandle( req );
+        WinHttpCloseHandle( con );
+        WinHttpCloseHandle( ses );
+        CloseHandle( info.wait );
+        return;
+    }
     ok(ret, "failed to send request %u\n", GetLastError());
 
     WaitForSingleObject( info.wait, INFINITE );
@@ -465,9 +549,22 @@ static void test_async( void )
     setup_test( &info, winhttp_close_handle, __LINE__ );
     WinHttpCloseHandle( req );
     WinHttpCloseHandle( con );
+
+    if (unload)
+    {
+        status = WaitForSingleObject( event, 0 );
+        ok(status == WAIT_TIMEOUT, "got %08x\n", status);
+    }
     WinHttpCloseHandle( ses );
 
     WaitForSingleObject( info.wait, INFINITE );
+
+    if (unload)
+    {
+        status = WaitForSingleObject( event, 2000 );
+        ok(status == WAIT_OBJECT_0, "got %08x\n", status);
+    }
+    CloseHandle( event );
     CloseHandle( info.wait );
 }
 
